@@ -1,5 +1,5 @@
 #' @importFrom dplyr %>%
-#' @importFrom tools file_path_sans_ext
+#' @importFrom stats dist
 NULL
 
 ################################################################################
@@ -9,9 +9,11 @@ NULL
 #' Train deconvolution model for spatial transcriptomics data
 #'
 #' Train a deep neural network model using training data from the
-#' \code{\linkS4class{SpatialDDLS}} object. In addition, the trained model is
-#' evaluated using test data, and prediction results are obtained to determine
-#' its performance (see \code{?\link{calculateEvalMetrics}}).
+#' \code{\linkS4class{SpatialDDLS}} object. This model will be used to 
+#' deconvolute spatial transcriptomics data from the same biological context as 
+#' the single-cell RNA-seq data used to train it. In addition, the trained 
+#' model is evaluated using test data, and prediction results are obtained to 
+#' determine its performance (see \code{?\link{calculateEvalMetrics}}).
 #'
 #' \strong{Simulation of mixed transcriptional profiles 'on the fly'}
 #'
@@ -35,25 +37,25 @@ NULL
 #'   \code{single.cell.real}/\code{single.cell.simul}, \code{prob.cell.types},
 #'   and \code{mixed.profiles} slots (the last only if \code{on.the.fly =
 #'   FALSE}).
-#' @param type.data.train Type of profiles used for training. It can be
+#' @param type.data.train Type of profiles to be used for training. It can be
 #'   \code{'both'}, \code{'single-cell'} or \code{'mixed'} (\code{'mixed'} by
 #'   default).
-#' @param type.data.test Type of profiles used for evaluation. It can be
+#' @param type.data.test Type of profiles to be used for evaluation. It can be
 #'   \code{'both'}, \code{'single-cell'} or \code{'mixed'} (\code{'mixed'} by
 #'   default).
 #' @param batch.size Number of samples per gradient update (64 by default).
-#' @param num.epochs Number of epochs to train the model (10 by default).
+#' @param num.epochs Number of epochs to train the model (60 by default).
 #' @param num.hidden.layers Number of hidden layers of the neural network (2 by
 #'   default). This number must be equal to the length of \code{num.units}
 #'   argument.
 #' @param num.units Vector indicating the number of neurons per hidden layer
 #'   (\code{c(200, 200)} by default). The length of this vector must be equal to
-#'   \code{num.hidden.layers} argument.
-#' @param activation.fun Activation function (\code{'sigmoid'} by default). See
+#'   the \code{num.hidden.layers} argument.
+#' @param activation.fun Activation function (\code{'relu'} by default). See
 #'   the
 #'   \href{https://tensorflow.rstudio.com/reference/keras/activation_relu.html}{keras
 #'    documentation} to know available activation functions.
-#' @param dropout.rate Float between 0 and 1 indicating the fraction of the
+#' @param dropout.rate Float between 0 and 1 indicating the fraction of
 #'   input neurons to be dropped in layer dropouts (0.25 by default). By
 #'   default, \pkg{SpatialDDLS} implements 1 dropout layer per hidden layer.
 #' @param loss Character indicating loss function selected for model training
@@ -65,38 +67,44 @@ NULL
 #'   "categorical_accuracy")} by default). See the
 #'   \href{https://tensorflow.rstudio.com/reference/keras/metric_binary_accuracy.html}{keras
 #'    documentation} to know available performance metrics.
-#' @param scaling How to scale data before training. It may be:
-#'   \code{"standarize"} (values are centered around the mean with a unit
-#'   standard deviation) or \code{"rescale"} (values are shifted and rescaled so
-#'   that they end up ranging between 0 and 1, by default).
+#' @param normalize Whether to normalize data using logCPM (\code{TRUE} by 
+#'   default). This parameter is only considered when the method used to 
+#'   simulate mixed transcriptional profiles (\code{simMixedProfiles} 
+#'   function) was \code{"AddRawCount"}. Otherwise, data were already 
+#'   normalized.
+#' @param scaling How to scale data before training. It can be:
+#'   \code{"standardize"} (values are centered around the mean with a unit
+#'   standard deviation), \code{"rescale"} (values are shifted and rescaled so
+#'   that they end up ranging between 0 and 1) or \code{"none"} (no
+#'   scaling is performed). \code{"standardize"} by default. 
 #' @param norm.batch.layers Whether to include batch normalization layers
-#'   between each hidden dense layer (\code{FALSE} by default).
-#' @param custom.model It allows to use a custom neural network. It must be a
-#'   \code{keras.engine.sequential.Sequential} object in which the number of
-#'   input neurons is equal to the number of considered features/genes, and the
-#'   number of output neurons is equal to the number of cell types considered
-#'   (\code{NULL} by default). If provided, the arguments related to the neural
-#'   network architecture will be ignored.
+#'   between each hidden dense layer (\code{TRUE} by default).
+#' @param custom.model It allows to use a custom neural network architecture. It 
+#'   must be a \code{keras.engine.sequential.Sequential} object in which the 
+#'   number of input neurons is equal to the number of considered 
+#'   features/genes, and the number of output neurons is equal to the number of 
+#'   cell types considered (\code{NULL} by default). If provided, the arguments 
+#'   related to the neural network architecture will be ignored.
 #' @param shuffle Boolean indicating whether data will be shuffled (\code{TRUE}
 #'   by default).
 #' @param sc.downsampling It is only used if \code{type.data.train} is equal to
 #'   \code{'both'} or \code{'single-cell'}. It allows to set a maximum number of
-#'   single-cell profiles from a specific cell type  used for training to avoid
-#'   an unbalanced representation of cell types (\code{NULL} by default).
+#'   single-cell profiles of a specific cell type for training to avoid
+#'   an unbalanced representation of classes (\code{NULL} by default).
 #' @param use.generator Boolean indicating whether to use generators during
 #'   training and test. Generators are automatically used when \code{on.the.fly
 #'   = TRUE} or HDF5 files are used, but it can be activated by the user on
 #'   demand (\code{FALSE} by default).
 #' @param on.the.fly Boolean indicating whether simulated data will be generated
 #'   'on the fly' during training (\code{FALSE} by default).
-#' @param agg.function Function used to build mixed transcriptional profiles. It
-#'   may be: \itemize{ \item \code{"AddRawCount"} (by default): single-cell
+#' @param agg.function If \code{on.the.fly == TRUE}, function used to build 
+#'   mixed transcriptional profiles. It may be: 
+#'   \itemize{ \item \code{"AddRawCount"} (by default): single-cell
 #'   profiles (raw counts) are added up across cells. Then, log-CPMs are
 #'   calculated. \item \code{"MeanCPM"}: single-cell profiles (raw counts) are
-#'   transformed into CPMs and cross-cell averages are calculated. Then,
-#'   \code{log2(CPM + 1)} is calculated. \item \code{"AddCPM"}: single-cell
-#'   profiles (raw counts) are transformed into CPMs and are added up across
-#'   cells. Then, log-CPMs are calculated.}
+#'   transformed into logCPM and cross-cell averages are calculated. 
+#'   \item \code{"AddCPM"}: single-cell profiles (raw counts) are transformed 
+#'   into CPMs and are added up across cells. Then, log-CPMs are calculated.}
 #' @param threads Number of threads used during simulation of mixed
 #'   transcriptional profiles if \code{on.the.fly = TRUE} (1 by default).
 #' @param view.metrics.plot Boolean indicating whether to show plots of loss and
@@ -138,18 +146,20 @@ NULL
 #'   sc.data = sce,
 #'   sc.cell.ID.column = "Cell_ID",
 #'   sc.gene.ID.column = "Gene_ID",
+#'   sc.filt.genes.cluster = FALSE
 #' )
 #' SDDLS <- genMixedCellProp(
 #'   object = SDDLS,
 #'   cell.ID.column = "Cell_ID",
 #'   cell.type.column = "Cell_Type",
 #'   num.sim.spots = 50,
+#'   train.freq.cells = 2/3,
+#'   train.freq.spots = 2/3,
 #'   verbose = TRUE
 #' )
 #' SDDLS <- simMixedProfiles(SDDLS)
 #' SDDLS <- trainDeconvModel(
 #'   object = SDDLS,
-#'   on.the.fly = TRUE,
 #'   batch.size = 12,
 #'   num.epochs = 5
 #' )
@@ -160,22 +170,23 @@ trainDeconvModel <- function(
   type.data.train = "mixed",
   type.data.test = "mixed",
   batch.size = 64,
-  num.epochs = 10,
+  num.epochs = 60,
   num.hidden.layers = 2,
   num.units = c(200, 200),
-  activation.fun = "sigmoid",
+  activation.fun = "relu",
   dropout.rate = 0.25,
   loss = "kullback_leibler_divergence",
   metrics = c("accuracy", "mean_absolute_error",
               "categorical_accuracy"),
-  scaling = "rescale",
-  norm.batch.layers = FALSE,
+  normalize = TRUE,
+  scaling = "standardize",
+  norm.batch.layers = TRUE,
   custom.model = NULL,
-  shuffle = FALSE,
+  shuffle = TRUE,
   sc.downsampling = NULL,
   use.generator = FALSE,
   on.the.fly = FALSE,
-  agg.function = "MeanCPM",
+  agg.function = "AddRawCount",
   threads = 1,
   view.metrics.plot = TRUE,
   verbose = TRUE
@@ -202,13 +213,15 @@ trainDeconvModel <- function(
   #        "or 'single.cell.simul') as trainDeconvModel evaluates ", 
   #        "DNN model on both types of profiles: mixed and single-cell")
   # }
-  if (!scaling %in% c("standarize", "rescale")) {
-    stop("'scaling' argument must be one of the following options: 'standarize', 'rescale'")
+  if (!scaling %in% c("standardize", "rescale", "none")) {
+    stop("'scaling' argument must be one of the following options: 'standardize', 'rescale' or 'none'")
   } else {
-    if (scaling == "standarize") {
+    if (scaling == "standardize") {
       scaling.fun <- base::scale
     } else if (scaling == "rescale") {
       scaling.fun <- rescale.function
+    } else if (scaling == "none") {
+      scaling.fun <- function(x) return(x)
     }
   }
   if (!on.the.fly) {
@@ -285,7 +298,7 @@ trainDeconvModel <- function(
   } 
   if (n.test < batch.size) {
     stop(
-      paste0("The number of samples used for training (", n.train, ") is too ",
+      paste0("The number of samples used for test (", n.test, ") is too ",
              "small compared with 'batch.size' (", batch.size, "). Please, ",
              "increase the number of samples or consider reducing 'batch.size'")
     )
@@ -395,15 +408,19 @@ trainDeconvModel <- function(
   if (verbose) 
     message(paste("\n=== Training DNN with", n.train, "samples:\n"))
   
-  
-  
   if (use.generator | isTRUE(on.the.fly) | checkingClass) {
     gen.train <- .trainGenerator(
       object = object, 
       funGen = .dataForDNN,
       prob.matrix = prob.matrix.train,
       type.data = "train",
+      mixing.fun = ifelse(
+        any(type.data.train %in% c("mixed", "both")) & on.the.fly == FALSE,
+        yes = mixed.profiles(object, "train")@metadata[["mixing.fun"]],
+        no = agg.function
+      ),
       fun.aggregation = .agg.fun,
+      normalize = normalize,
       scaling = scaling.fun,
       batch.size = batch.size,
       combine = type.data.train,
@@ -430,7 +447,13 @@ trainDeconvModel <- function(
       funGen = .dataForDNN,
       target = TRUE,
       prob.matrix = prob.matrix.test,
+      mixing.fun = ifelse(
+        any(type.data.test %in% c("mixed", "both")) & on.the.fly == FALSE,
+        yes = mixed.profiles(object, "test")@metadata[["mixing.fun"]],
+        no = agg.function
+      ),
       fun.aggregation = .agg.fun,
+      normalize = normalize,
       scaling = scaling.fun,
       batch.size = batch.size,
       pattern = pattern,
@@ -452,7 +475,13 @@ trainDeconvModel <- function(
       funGen = .dataForDNN,
       target = FALSE,
       prob.matrix = prob.matrix.test,
+      mixing.fun = ifelse(
+        any(type.data.test %in% c("mixed", "both")) & on.the.fly == FALSE,
+        yes = mixed.profiles(object, "test")@metadata[["mixing.fun"]],
+        no = agg.function
+      ),
       fun.aggregation = .agg.fun,
+      normalize = normalize,
       scaling = scaling.fun,
       batch.size = batch.size,
       pattern = pattern,
@@ -472,7 +501,13 @@ trainDeconvModel <- function(
       sel.data = prob.matrix.train,
       pattern = pattern,
       type.data = "train",
+      mixing.fun = ifelse(
+        any(type.data.train %in% c("mixed", "both")) & on.the.fly == FALSE,
+        yes = mixed.profiles(object, "train")@metadata[["mixing.fun"]],
+        no = agg.function
+      ),
       fun.aggregation = .agg.fun,
+      normalize = normalize,
       scaling = scaling.fun,
       threads = threads
     )
@@ -493,7 +528,13 @@ trainDeconvModel <- function(
       sel.data = prob.matrix.test,
       pattern = pattern,
       type.data = "test",
+      mixing.fun = ifelse(
+        any(type.data.test %in% c("mixed", "both")) & on.the.fly == FALSE,
+        yes = mixed.profiles(object, "test")@metadata[["mixing.fun"]],
+        no = agg.function
+      ),
       fun.aggregation = .agg.fun,
+      normalize = normalize,
       scaling = scaling.fun,
       threads = threads
     )
@@ -535,7 +576,9 @@ trainDeconvModel <- function(
   funGen,
   prob.matrix,
   type.data,
+  mixing.fun,
   fun.aggregation,
+  normalize,
   scaling,
   batch.size,
   combine,
@@ -572,7 +615,9 @@ trainDeconvModel <- function(
         sel.data = sel.data, 
         pattern = pattern,
         type.data = type.data,
+        mixing.fun = mixing.fun,
         fun.aggregation = fun.aggregation,
+        normalize = normalize,
         scaling = scaling,
         threads = threads
       )
@@ -587,7 +632,9 @@ trainDeconvModel <- function(
         sel.data = sel.data, 
         pattern = pattern,
         type.data = type.data,
+        mixing.fun = mixing.fun,
         fun.aggregation = fun.aggregation,
+        normalize = normalize,
         scaling = scaling,
         threads = threads
       )
@@ -601,7 +648,9 @@ trainDeconvModel <- function(
   funGen,
   prob.matrix,
   target,
+  mixing.fun,
   fun.aggregation,
+  normalize,
   scaling,
   batch.size,
   pattern,
@@ -623,7 +672,9 @@ trainDeconvModel <- function(
       sel.data = sel.data, 
       pattern = pattern,
       type.data = "test",
+      mixing.fun = mixing.fun,
       fun.aggregation = fun.aggregation,
+      normalize = normalize,
       scaling = scaling,
       threads = threads
     )
@@ -631,13 +682,15 @@ trainDeconvModel <- function(
     else return(list(counts))
   }
 }
-
+## in this function, I have to normalize raw counts
 .dataForDNN.file <- function(
   object,
   sel.data,
   pattern,
   type.data,
+  mixing.fun,
   fun.aggregation,
+  normalize,
   scaling,
   threads
 ) {
@@ -677,8 +730,14 @@ trainDeconvModel <- function(
   # return final matrix counts
   if (any(bulk.data) && any(!bulk.data)) {
     cell.samples <- log2(.cpmCalculate(x = cell.samples + 1))
+    if (normalize & mixing.fun == "AddRawCount") {
+      bulk.samples <- log2(.cpmCalculate(x = bulk.samples + 1))
+    }
     counts <- cbind(bulk.samples, cell.samples)[, rownames(sel.data), drop = FALSE]
   } else if (any(bulk.data)) {
+    if (normalize & mixing.fun == "AddRawCount") {
+      bulk.samples <- log2(.cpmCalculate(x = bulk.samples + 1))
+    }
     counts <- bulk.samples[, rownames(sel.data), drop = FALSE]
   } else if (any(!bulk.data)) {
     counts <- log2(
@@ -693,7 +752,9 @@ trainDeconvModel <- function(
   sel.data,
   pattern,
   type.data,
+  mixing.fun,
   fun.aggregation,
+  normalize,
   scaling,
   threads
 ) {
@@ -739,8 +800,14 @@ trainDeconvModel <- function(
   # return final matrix counts
   if (any(bulk.data) && any(!bulk.data)) {
     cell.samples <- log2(.cpmCalculate(x = cell.samples + 1))
+    if (normalize & mixing.fun == "AddRawCount") {
+      bulk.samples <- log2(.cpmCalculate(x = bulk.samples + 1))
+    }
     counts <- cbind(bulk.samples, cell.samples)[, rownames(sel.data), drop = FALSE]
   } else if (any(bulk.data)) {
+    if (normalize & mixing.fun == "AddRawCount") {
+      bulk.samples <- log2(.cpmCalculate(x = bulk.samples + 1))
+    }
     counts <- bulk.samples[, rownames(sel.data), drop = FALSE]
   } else if (any(!bulk.data)) {
     counts <- log2(
@@ -1023,10 +1090,49 @@ trainDeconvModel <- function(
 ############### Deconvolution of spatial transcriptomics data ##################
 ################################################################################
 
-#' Deconvolute spatial transcriptomics data
+#' Deconvolute spatial transcriptomics data using trained model
 #'
-#' Deconvolute spatial transcriptomics data using the trained model contained in
-#' the \code{\linkS4class{SpatialDDLS}} object.
+#' Deconvolute spatial transcriptomics data using the trained model in
+#' the \code{\linkS4class{SpatialDDLS}} object. The trained model is used 
+#' to predict cell proportions of two mirrored transcriptional profiles: 
+#' \itemize{ \item 'Intrinsic' profiles: transcriptional profiles of each spot 
+#' in the ST dataset. \item 'Extrinsic' profiles: profiles simulated from the 
+#' surrounding spots of each spot.} After prediction, cell proportions
+#' from the intrinsic profiles (intrinsic cell proportions) are regularized 
+#' based on the similarity between intrinsic and extrinsic profiles in order
+#' to maintain spatial consistency. This approach leverages both transcriptional 
+#' and spatial information. For more details, see Mañanes et al., 2023 and the
+#' Details section. 
+#' 
+#' The deconvolution process involves two main steps: predicting cell 
+#' proportions based on transcriptome using the trained neural network model, 
+#' and regularization of cell proportions based on the spatial location of each 
+#' spot. In the regularization step, a mirrored version of each spot is 
+#' simulated based on its N-nearest spots. We refer to these profiles as 
+#' 'extrinsic' profiles, whereas the transcriptional profiles of each spot are 
+#' called 'intrinsic' profiles. Extrinsic profiles are used to regularize 
+#' predictions based on intrinsic profiles. The rationale is that spots 
+#' surrounded by transcriptionally similar spots should have similar cell 
+#' compositions, and therefore predicted proportions can be smoothed to preserve
+#' their spatial consistency. On the other hand, spots surrounded by dissimilar 
+#' spots cannot be predicted by their neighbors, and thus they can only be 
+#' predicted by their own transcriptional profiles likely due to presenting very 
+#' specific cell compositions. 
+#' 
+#' Regarding the working os \pkg{SpatialDDLS}: first, extrinsic profiles are 
+#' simulated based on the N-nearest spots for each spot by summing their 
+#' transcriptomes. Distances between extrinsic and intrinsic profiles of each 
+#' spot are calculated so that similar/dissimilar spots are identified. These 
+#' two sets of transcriptional profiles are used as input for the trained neural
+#' network model, and according to the calculated distances, a weighted mean 
+#' between the predicted proportions for each spot is calculated. Spots with 
+#' distances between intrinsic and extrinsic profiles greater than 
+#' \code{alpha.cutoff} are not regularized, whereas spots with distances less 
+#' than \code{alpha.cutoff} contribute to the weighted mean. Weights are 
+#' calculated by rescaling distances less than \code{alpha.cutoff} between 0 
+#' and 0.5, so that the maximum extent to which a extrinsic profile can 
+#' modified the predictions based on intrinsic profiles is 0.5 (a regular 
+#' mean). For more details, see Mañanes et al., 2023. 
 #'
 #' This function requires a \code{\linkS4class{SpatialDDLS}} object with a
 #' trained deep neural network model (\code{\link{trained.model}} slot, and the
@@ -1039,13 +1145,37 @@ trainDeconvModel <- function(
 #' @param index.st Name or index of the dataset/slide stored in the
 #'   \code{SpatialDDLS} object (\code{spatial.experiments} slot) to be
 #'   deconvolute. If missing, all datasets will be deconvoluted.
-#' @param normalize Normalize data before deconvolution (\code{TRUE} by
-#'   default).
-#' @param scaling How to scale data before training. It may be:
-#'   \code{"standarize"} (values are centered around the mean with a unit
+#' @param normalize Normalize data (logCPM) before deconvolution (\code{TRUE} by
+#'   default). 
+#' @param scaling How to scale data before training. Options include
+#'   \code{"standardize"} (values are centered around the mean with a unit
 #'   standard deviation) or \code{"rescale"} (values are shifted and rescaled so
 #'   that they end up ranging between 0 and 1). If \code{normalize = FALSE},
 #'   data are not scaled.
+#' @param k.spots Number of nearest spots considered for each spot during 
+#'   regularization and simulation of extrinsic transcriptional profiles. The 
+#'   greater, the smoother the regularization will be (4 by default). 
+#' @param pca.space Whether to use PCA space to calculate distances between 
+#'   intrinsic and extrinsic transcriptional profiles (\code{TRUE} by default). 
+#' @param fast.pca Whether using the \pkg{irlba} implementation. If \code{TRUE},
+#'   the number of PCs used is defined by the \code{} parameter. If 
+#'   \code{FALSE}, the PCA implementation from the \pkg{stats} R package is 
+#'   used instead (\code{TRUE} by default).
+#' @param pcs.num Number of PCs used to calculate distances if 
+#'   \code{fast.pca == TRUE} (50 by default). 
+#' @param pca.var Threshold of explained 
+#'   variance (between 0.2 and 1) used to choose the number of PCs used if 
+#'   \code{pca.space == TRUE} and \code{fast.pca == FALSE} (0.8 by default). 
+#' @param metric Metric used to measure distance/similarity between intrinsic 
+#'   and extrinsic transcriptional profiles. It may be \code{'euclidean'}, 
+#'   \code{'cosine'} or \code{'pearson'} (\code{'euclidean'} by default).
+#' @param alpha.cutoff Minimum distance for regularization. 
+#'   It may be \code{'mean'} (spots with transcriptional distances shorter than 
+#'   the mean distance of the dataset will be modified) or \code{'quantile'} 
+#'   (spots with transcriptional distances shorter than the 
+#'   \code{alpha.quantile} quantile are used). \code{'mean'} by default.
+#' @param alpha.quantile Quantile used if \code{alpha.cutoff == 'quantile'}. 
+#'   0.5 by default.  
 #' @param simplify.set List specifying which cell types should be compressed
 #'   into a new label with the name of the list item. See examples for details.
 #'   If provided, results are stored in a list with \code{'raw'} and
@@ -1062,11 +1192,13 @@ trainDeconvModel <- function(
 #' @param verbose Show informative messages during the execution.
 #'
 #' @return \code{\linkS4class{SpatialDDLS}} object with a \code{deconv.spots}
-#'   slot. The output is a data frame with spot (\eqn{i}) as rows and cell types
-#'   (\eqn{j}) as columns. Each entry represents the predicted proportion of
-#'   \eqn{j} cell type in \eqn{i} spot. If \code{simplify.set} and/or
-#'   \code{simplify.majority} are provided, the \code{deconv.spots} slot will
-#'   contain a list with raw and simplified results.
+#'   slot. The output is a list containing 'Regularized', 'Intrinsic' and 
+#'   'Extrinsic' deconvoluted cell proportions, 'Distances' between intrinsic 
+#'   and extrinsic transcriptional profiles, and 'Weight.factors' with the 
+#'   final weights used to regularize intrinsic cell proportions. If 
+#'   \code{simplify.set} and/or \code{simplify.majority} are provided, 
+#'   the \code{deconv.spots} slot will contain a list with raw and simplified 
+#'   results.
 #'
 #' @export
 #'
@@ -1095,12 +1227,15 @@ trainDeconvModel <- function(
 #'   sc.data = sce,
 #'   sc.cell.ID.column = "Cell_ID",
 #'   sc.gene.ID.column = "Gene_ID",
+#'   sc.filt.genes.cluster = FALSE
 #' )
 #' SDDLS <- genMixedCellProp(
 #'   object = SDDLS,
 #'   cell.ID.column = "Cell_ID",
 #'   cell.type.column = "Cell_Type",
-#'  num.sim.spots = 50,
+#'   num.sim.spots = 50,
+#'   train.freq.cells = 2/3,
+#'   train.freq.spots = 2/3,
 #'   verbose = TRUE
 #' ) 
 #' SDDLS <- simMixedProfiles(SDDLS)
@@ -1110,9 +1245,9 @@ trainDeconvModel <- function(
 #'   batch.size = 15,
 #'   num.epochs = 5
 #' )
-#' # simulating mixed profiles
+#' # simulating spatial data
 #' ngenes <- sample(3:40, size = 1)
-#' ncells <- sample(3:40, size = 1)
+#' ncells <- sample(10:40, size = 1)
 #' counts <- matrix(
 #'   rpois(ngenes * ncells, lambda = 5), ncol = ncells,
 #'   dimnames = list(paste0("Gene", seq(ngenes)), paste0("Spot", seq(ncells)))
@@ -1138,16 +1273,28 @@ trainDeconvModel <- function(
 #' SDDLS <- deconvSpatialDDLS(
 #'   object = SDDLS,
 #'   index.st = 1,
-#'   simplify.set = simplify,
+#'   simplify.set = simplify, 
 #'   simplify.majority = simplify
 #' )
 #' }
+#' @references Mañanes, D., Rivero-García, I., Jimenez-Carretero, D., 
+#'   Torres, M., Sancho, D., Torroja, C., Sánchez-Cabo, F. (2023). SpatialDDLS: 
+#'   An R package to deconvolute spatial transcriptomics data using neural 
+#'   networks. biorxiv. doi: \doi{10.1101/2023.08.31.555677}.
 #'   
 deconvSpatialDDLS <- function(
   object,
   index.st,
   normalize = TRUE,
-  scaling = "rescale",
+  scaling = "standardize",
+  k.spots = 4, 
+  pca.space = TRUE,
+  fast.pca = TRUE,
+  pcs.num = 50,
+  pca.var = 0.8,
+  metric = "euclidean",
+  alpha.cutoff = "mean",
+  alpha.quantile = 0.5,
   simplify.set = NULL,
   simplify.majority = NULL,
   use.generator = FALSE,
@@ -1163,15 +1310,28 @@ deconvSpatialDDLS <- function(
   } else if (is.null(spatial.experiments(object))) {
     stop("No spatial transcriptomics data provided. See ?createSpatialDDLSobject or ?loadSTProfiles")
   }
-  if (!scaling %in% c("standarize", "rescale")) {
-    stop("'scaling' argument must be one of the following options: 'standarize' or 'rescale'")
+  if (!scaling %in% c("standardize", "rescale", "none")) {
+    stop("'scaling' argument must be one of the following options: 'standardize' or 'rescale'")
   } else {
-    if (scaling == "standarize") {
+    if (scaling == "standardize") {
       scaling.fun <- base::scale
     } else if (scaling == "rescale") {
       scaling.fun <- rescale.function
+    } else if (scaling == "none") {
+      scaling.fun <- function(x) return(x)
     }
   }
+  ## checking parameters for regularization
+  if (pca.var > 1 | pca.var <= 0.2) {
+    stop("pca.var must be a doublet between 0.2 and 1")
+  }
+  if (!any(metric %in% c("euclidean", "cosine", "pearson"))) {
+    stop("metric only can be one of the following options: 'euclidean', 'cosine', 'pearson'")
+  }
+  if (!any(alpha.cutoff %in% c("mean", "quantile"))) {
+    stop("alpha.cutoff only can be 'mean' or 'quantile'")
+  }
+    
   # checking if DNN model is json format or compiled
   if (is.list(trained.model(object)@model)) {
     model.comp <- .loadModelFromJSON(trained.model(object))
@@ -1212,15 +1372,38 @@ deconvSpatialDDLS <- function(
         use.generator = use.generator,
         verbose = verbose
       )
+      ## spatial consistency: add here the function (with a conditional I guess)
+      list.res.post <- .spatialRegularization(
+        spatial.experiments(object, index.st = index), 
+        prop.intrinsic = results,
+        model = trained.model(object),
+        batch.size = batch.size,
+        normalize = normalize,
+        scaling = scaling.fun,
+        use.generator = use.generator,
+        k = k.spots, 
+        pca = pca.space,
+        fast.pca = fast.pca,
+        pcs.num = pcs.num,
+        pca.var = pca.var,
+        metric = metric,
+        alpha.cutoff = alpha.cutoff,
+        alpha.quantile = alpha.quantile,
+        verbose = verbose
+      )
+      list.res.post[["Instrinsic.Proportions"]] <- results
+      ## simplfy: not very used to be honest, let's keep it just for convenience 
       if (!is.null(simplify.set) || !is.null(simplify.majority)) {
-        list.res <- list(raw = results)
+        if (verbose) message("=== Note that only regularized proportions will be simplified")
+        
+        list.res <- list(raw = list.res.post)
         if (!is.null(simplify.set)) {
           if (!is(simplify.set, "list")) {
             stop("'simplify.set' must be a list in which each element is a ", 
                  "cell type considered by the model")
           }
           results.set <- .simplifySetGeneral(
-            results = results,
+            results = list.res.post[["Regularized"]],
             simplify.set = simplify.set
           )
           list.res[["simpli.set"]] <- results.set
@@ -1231,18 +1414,17 @@ deconvSpatialDDLS <- function(
                  "cell type considered by the model")
           }
           results.maj <- .simplifyMajorityGeneral(
-            results = results,
+            results = list.res.post[["Regularized"]],
             simplify.majority = simplify.majority
           )
           list.res[["simpli.majority"]] <- results.maj
         }
         return(list.res)
       } 
-      return(results)
+      return(list.res.post)
     }
   ) %>% setNames(namesList)
   deconv.spots(object) <- deconv.st
-  
   if (verbose) message("DONE")
   
   return(object)
@@ -1265,8 +1447,6 @@ deconvSpatialDDLS <- function(
   # filtering features missing in training data
   filter.features <- rownames(deconv.counts) %in% features(model)
   deconv.counts <- deconv.counts[filter.features, ]
-  # TODO: remove messages when there are 0 missed genes
-  # set features missing in spatial.experiments
   fill.features <- !features(model) %in% rownames(deconv.counts)
   if (any(fill.features)) {
     m.new <- matrix(0L, nrow = sum(fill.features), ncol = ncol(deconv.counts))
@@ -1283,7 +1463,7 @@ deconvSpatialDDLS <- function(
         )
       )  
     }
-    if (sum(fill.features) == 0) {
+    if (sum(fill.features) != 0) {
       message(
         paste(
           "=== Setting", sum(fill.features), 
@@ -1293,11 +1473,15 @@ deconvSpatialDDLS <- function(
     }
   }
   if (normalize) {
-    if (verbose) message("=== Normalizing and scaling data\n")
+    if (verbose) message("=== Normalizing data (LogCPM)\n")
+    
     deconv.counts <- log2(.cpmCalculate(x = deconv.counts + 1))
-    # TODO: here is the problem, check if just in case
+    deconv.counts <- scaling(t(deconv.counts))
+  } else {
+    deconv.counts <- as.matrix(deconv.counts)
     deconv.counts <- scaling(t(deconv.counts))
   }
+  
   if (verbose) {
     verbose.model <- 1
     message("=== Predicting cell type proportions\n")
@@ -1355,6 +1539,181 @@ deconvSpatialDDLS <- function(
   }
 }
 
+.spatialRegularization <- function(
+    st.data, 
+    prop.intrinsic,
+    model,
+    scaling,
+    batch.size = batch.size,
+    normalize = normalize,
+    use.generator = use.generator,
+    k = k, 
+    pca = pca,
+    fast.pca = fast.pca,
+    pcs.num = pcs.num,
+    pca.var = pca.var,
+    metric = metric,
+    alpha.cutoff = alpha.cutoff,
+    alpha.quantile = alpha.quantile,
+    verbose = verbose
+) {
+  ## calculating kNN spots per spot
+  distances <- as.matrix(dist(SpatialExperiment::spatialCoords(st.data)))
+  knn.indexes <- FNN::get.knn(distances, k = k)[[1]]
+  # print(knn.indexes)
+  rownames(knn.indexes) <- rownames(SpatialExperiment::spatialCoords(st.data))
+  
+  ## generating mixed transcriptional profiles using kNN spots
+  summ.nn.expr <- apply(
+    X = knn.indexes, MARGIN = 1, 
+    FUN = \(idx) Matrix::rowSums(SummarizedExperiment::assay(st.data)[, idx])
+  )
+  colnames(summ.nn.expr) <- colnames(SummarizedExperiment::assay(st.data))
+  ## prediction of cell proportions on extrinsic signals
+  prop.nn <-.deconvCore(
+    deconv.counts = summ.nn.expr,
+    model = model,
+    batch.size = batch.size,
+    normalize = normalize,
+    scaling = scaling,
+    use.generator = use.generator,
+    verbose = FALSE
+  )
+  ## normalizing both data in order to calculate distances 
+  logcpm.nn.expr <- log2(.cpmCalculate(summ.nn.expr + 1))
+  logcpm.spot.expr <- log2(
+    .cpmCalculate(
+      SummarizedExperiment::assay(st.data) + 1
+    )
+  )
+  ## calculating distances between actual and contextual matrices
+  if (isTRUE(pca)) {
+    if (verbose) message("\n=== Calculating distances in PCA space")
+    
+    logcpm.nn.expr.2 <- logcpm.nn.expr
+    colnames(logcpm.nn.expr.2) <- paste0(colnames(logcpm.nn.expr), "_NN")
+    
+    if (fast.pca) {
+      if (!requireNamespace("irlba", quietly = TRUE)) {
+        warning(
+          paste(
+            "irlba is required but not available. Running", 
+            "PCA implementation from the stats R package"
+          ), call. = FALSE, immediate. = TRUE
+        )
+      }
+      if (verbose) message("\n=== Calculating ", pcs.num, " PCs\n")
+      
+      log.cpm.final <- cbind(logcpm.spot.expr, logcpm.nn.expr.2)
+      if (pcs.num >= min(ncol(log.cpm.final), nrow(log.cpm.final))) {
+        pcs.num <- ceiling(min(nrow(log.cpm.final), ncol(log.cpm.final)) * 0.8)
+      }
+      pca.space <- irlba::irlba(A = t(x = log.cpm.final), nv = pcs.num)
+      pca.nn <- pca.space[[2]][grepl("_NN$", colnames(log.cpm.final)), ]
+      pca.ori <- pca.space[[2]][!grepl("_NN$", colnames(log.cpm.final)), ]
+    } else {
+      pca.space <- stats::prcomp(
+        t(cbind(logcpm.spot.expr, logcpm.nn.expr.2)), center = TRUE, scale. = TRUE
+      )
+      eigs <- pca.space$sdev^2
+      n.pcs <- sum(cumsum(eigs / sum(eigs)) <= pca.var)
+      if (verbose) 
+        message(paste0("        - Using ", n.pcs, " PCs (variance cutoff: ", pca.var, ")\n"))
+        
+      pca.nn <- pca.space$x[grep("_NN$", rownames(pca.space$x)), 1:n.pcs]
+      pca.ori <- pca.space$x[grep("_NN$", rownames(pca.space$x), invert = T), 1:n.pcs]
+    }
+    ## distances
+    dist.mm <- .distCalc(x = pca.ori, y = pca.nn, metric = metric)
+  } else {
+    if (verbose) message("\n=== Calculating distances in transcriptome space\n")
+    
+    dist.mm <- .distCalc(x = logcpm.spot.expr, y = logcpm.nn.expr, metric = metric)
+  }
+  ## calculating alpha regularization based on distances
+  if (verbose) message("=== Calculating alpha factors based on distances\n")
+  
+  if (alpha.cutoff == "mean") {
+    dist.mod <- as.vector(dist.mm) <= mean(as.vector(dist.mm))
+    dist.rescaled <- rescale.function.2(
+      as.vector(dist.mm)[dist.mod], 
+      to = c(0, 0.5)
+    )  
+  } else if (alpha.cutoff == "quantile") {
+    if (alpha.quantile > 0.99 | alpha.quantile < 0.01) {
+      stop("alpha.quantile only can be a number between 0.99 and 0.01")
+    }
+    dist.mod <- as.vector(dist.mm) <= stats::quantile(
+      x = as.vector(dist.mm), probs = alpha.quantile
+    )
+    dist.rescaled <- rescale.function.2(
+      as.vector(dist.mm)[dist.mod], 
+      to = c(0, 0.5)
+    ) 
+  }
+  alpha.extrin <- ifelse(test = dist.mod, yes = dist.rescaled, no = 0)
+  alpha.intric <- 1 - alpha.extrin 
+  
+  ## probably, this part can be solved using matrix multiplication, check in the future
+  regulaized.prop <- matrix(0, nrow = nrow(prop.intrinsic), ncol = ncol(prop.intrinsic))
+  for (spot in seq(nrow(prop.intrinsic))) {
+    regulaized.prop[spot, ] <- sapply(
+      X = colnames(prop.intrinsic), 
+      FUN = \(x) {
+        stats::weighted.mean(
+          c(prop.intrinsic[spot, x], prop.nn[spot, x]), 
+          w = c(alpha.intric[spot], alpha.extrin[spot])
+        )
+      }
+    )  
+  }
+  colnames(regulaized.prop) <- colnames(prop.intrinsic)
+  rownames(regulaized.prop) <- rownames(prop.intrinsic)
+  return(
+    list(
+      regulaized.prop, prop.intrinsic, prop.nn, as.vector(dist.mm), alpha.extrin
+    ) %>% setNames(
+        c("Regularized", "Intrinsic", 
+          "Extrinsic", "Distances", "Alpha.Factors")
+      )
+  )
+}
+
+.distCalc <- function(x, y, metric) {
+  dist.mm <- matrix(0, ncol = 1, nrow = nrow(x))
+  if (metric == "euclidean") {
+    for (i in seq(nrow(x))) {
+      dist.mm[i, 1] <- stats::dist(
+        t(cbind(x[i, ], y[i, ])), method = "euclidean"
+      )
+    }  
+  } else if (metric == "cosine") {
+    for (i in seq(nrow(x))) {
+      if (!requireNamespace("lsa", quietly = TRUE)) {
+        stop("lsa R package is required but not available")
+      }
+      dist.mm[i, 1] <- lsa::cosine(x = x[i, ], y = y[i, ])
+    } 
+    dist.mm <- 1 - dist.mm ## from similarity to distance
+  } else if (metric == "pearson") {
+    for (i in seq(nrow(x))) {
+      dist.mm[i, 1] <- stats::cor(
+        x = x[i, ], y = y[i, ], method = "pearson"
+      )
+    } 
+    dist.mm[dist.mm < 0] <- 0
+    dist.mm <- 1 - dist.mm 
+  }
+  
+  return(dist.mm)
+}
+
+rescale.function.2 <- function(
+    x, to = c(0, 1), from = range(x, na.rm = TRUE, finite = TRUE)
+) {
+  (x - from[1]) / diff(from) * diff(to) + to[1]
+}
+
 rescale.function <- function(x) {
   apply(X = x, MARGIN = 2, FUN = function(x) (x - min(x)) / (max(x) - min(x)))
 }
